/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "PrecisePolygon3D.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

PrecisePolygon3D::PrecisePolygon3D() {
}



PrecisePolygon3D::PrecisePolygon3D(const PrecisePolygon3D &poly) {
	LengthUnit unit = MILLIMETER;
	for (unsigned int i = 0; i < poly.list.size(); ++i) {
		Point3D point = poly.list[i];
		list.push_back(Point3D(point.getX(unit), point.getY(unit), point.getZ(unit), unit));
	}
}



PrecisePolygon3D::~PrecisePolygon3D() {
}



std::string PrecisePolygon3D::getBaseTag() {
	return "PRECISEPOLYGON";
}



std::string PrecisePolygon3D::getTemplate() {
	return "<" + getBaseTag() + ">" + "<GENERATOR/>"
		+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}



void PrecisePolygon3D::fillInto(xmltio::Location & loc) const {
	Type::fillInto(loc);

	int n = list.size();
	for (int i = 0; i < n; i++) {
		Location item(Point3D::getTemplate(), Point3D::getBaseTag());
		item = list[i];
		loc.add(item);
	}
}



void PrecisePolygon3D::fromLocation(const xmltio::Location & loc, PrecisePolygon3D & pol) {
	Type::fromLocation(loc, pol);

	for (XPathIterator i = XPath(Point3D::getBaseTag()).evaluate(loc); i; ++i) {
		Location pointLocation(*i);
		Point3D p = extract<Point3D>(pointLocation);
		pol.addPoint(p);
	}
}



void PrecisePolygon3D::sanitizeLocation(xmltio::Location & loc) const {
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}


bool PrecisePolygon3D::operator ==(const PrecisePolygon3D & other) const {
	try {
		if (list.size() != other.list.size())
			return false;

		for (unsigned int i = 0; i < list.size(); i++) {
			if (list[i].getX(MILLIMETER) != other.list[i].getX(MILLIMETER)
					|| list[i].getY(MILLIMETER) != other.list[i].getY(MILLIMETER)
					|| list[i].getZ(MILLIMETER) != other.list[i].getZ(MILLIMETER)) {
				return false;
			}
		}
	} catch (exception &e) {
		return false;
	}
	return true;
}



Point3D PrecisePolygon3D::getCentroid() {
	float x = 0;
	float y = 0;
	float z = 0;
	int count = 0;
	for (unsigned int i = 0; i < list.size(); ++i) {
		Point3D &p = list[i];
		x += p.getX(MILLIMETER);
		y += p.getY(MILLIMETER);
		z += p.getZ(MILLIMETER);
		count++;
	}

	Point3D point(0, 0, 0, MILLIMETER);

	if (count != 0) {
		point.setX(x / (double)count, MILLIMETER);
		point.setY(y / (double)count, MILLIMETER);
		point.setZ(z / (double)count, MILLIMETER);
	}

	return point;
}



void PrecisePolygon3D::addPoint(double x, double y, double z, LengthUnit unit) {
	list.push_back(Point3D(x, y, z, unit));
}



void PrecisePolygon3D::addPoint(Point3D point) {
	list.push_back(point);
}


