/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * Door.cpp
 *
 *  Created on: Aug 18, 2009
 *      Author: jwienke
 */

#include "Door.h"

#include <iostream>
#include <xmltio/xmltio.hpp>

using namespace std;
using namespace xmltio;
using namespace airobots;

Door::Door() :
		 confidence(0), cs(RobotCS) {
}

Door::~Door() {
}

string Door::getBaseTag() {
	return "DOOR";
}

string Door::getTemplate() {

	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + "<LEFT>" + Point2D::getTemplate()
			+ "</LEFT><RIGHT>" + Point2D::getTemplate() + "</RIGHT><HANDLELEFT>"
			+ Point3D::getTemplate() + "</HANDLELEFT><HANDLERIGHT>"
			+ Point3D::getTemplate() + "</HANDLERIGHT><CONFIDENCE/>"
			+ "<COORDINATE_SYSTEM />" + "</" + getBaseTag() + ">";
}

string Door::getPerceptName() {
	return "door";
}

Point2D airobots::Door::getDoorLeft() const {
	return doorLeft;
}

void airobots::Door::setDoorLeft(Point2D door) {
	doorLeft = door;
}

Point2D airobots::Door::getDoorRight() const {
	return doorRight;
}

void airobots::Door::setDoorRight(Point2D door) {
	this->doorRight = door;
}

Point3D airobots::Door::getHandleLeft() const {
	return handleLeft;
}

void airobots::Door::setHandleLeft(Point3D handle) {
	this->handleLeft = handle;
}

Point3D airobots::Door::getHandleRight() const {
	return handleRight;
}

void airobots::Door::setHandleRight(Point3D handle) {
	this->handleRight = handle;
}

float airobots::Door::getConfidence() const {
	return confidence;
}
void airobots::Door::setConfidence(float confidence) {
	this->confidence = confidence;
}

airobots::Door::CoordinateSystem airobots::Door::getCoordinateSystem() const {
	return cs;
}

void airobots::Door::setCoordinateSystem(airobots::Door::CoordinateSystem cs) {
	this->cs = cs;
}

bool Door::operator==(const Door &other) const {
	return Type::operator==(other) && (other.doorLeft == doorLeft)
			&& (other.doorRight == doorRight)
			&& (other.handleLeft == handleLeft)
			&& (other.handleRight == handleRight) && (other.cs == cs);
}

std::ostream& airobots::operator<<(std::ostream& o, Door const& door) {
	return o << "Door [left = " << door.getDoorLeft() << ", right = "
			<< door.getDoorRight() << ", handle left = " << door.getHandleLeft()
			<< ", handle right = " << door.getHandleRight()
			<< ", confidence = " << door.getConfidence()
			<< ", coordinate system = " << door.getCoordinateSystem() << "]";
}

void Door::sanitizeLocation(xmltio::Location &loc) const {

}

void Door::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	Location l0 = loc[XPath("LEFT/" + Point2D::getBaseTag())];
	doorLeft.fillInto(l0);
	Location l1 = loc[XPath("RIGHT/" + Point2D::getBaseTag())];
	doorRight.fillInto(l1);
	Location l2 = loc[XPath("HANDLELEFT/" + Point3D::getBaseTag())];
	handleLeft.fillInto(l2);
	Location l3 = loc[XPath("HANDLERIGHT/" + Point3D::getBaseTag())];
	handleRight.fillInto(l3);

	loc[XPath("CONFIDENCE")] = confidence;
	loc[XPath("COORDINATE_SYSTEM")] = Door::csToString(cs);
}

void Door::fromLocation(const xmltio::Location &loc_, Door &door) {

	Type::fromLocation(loc_, door);

	Point2D left;
	Point2D::fromLocation(loc_[XPath("LEFT/" + Point2D::getBaseTag())], left);
	door.setDoorLeft(left);

	Point2D right;
	Point2D::fromLocation(loc_[XPath("RIGHT/" + Point2D::getBaseTag())], right);
	door.setDoorRight(right);

	Point3D handleleft;
	Point3D::fromLocation(loc_[XPath("HANDLELEFT/" + Point3D::getBaseTag())],
			handleleft);
	door.setHandleLeft(handleleft);

	Point3D handleright;
	Point3D::fromLocation(loc_[XPath("HANDLERIGHT/" + Point3D::getBaseTag())],
			handleright);
	door.setHandleRight(handleright);

	door.setConfidence(
				atof(loc_[XPath("CONFIDENCE")].getTextValue().c_str()));
	door.setCoordinateSystem(
			csFromString(loc_[XPath("COORDINATE_SYSTEM")].getTextValue()));
}
