/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include "FaceList.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

FaceList::FaceList() {
	faces = std::vector<Face>();
}

FaceList::~FaceList() {
}

std::string FaceList::getTemplate()
{
	return "<" + getBaseTag() + "/>";
}



void FaceList::fromLocation(const xmltio::Location & loc, FaceList & type)
{
	for (XPathIterator i = XPath(getBaseTag()).evaluate(loc); i; ++i) {
		Location l(*i);
		Face face = extract<Face>(l);
		type.addFace(face);
	}
}



void FaceList::fillInto(xmltio::Location & loc) const
{
	loc[XPath("@creator")] = getGenerator();

	vector<Face>::const_iterator i = faces.begin();
	for (; i != faces.end(); ++i) {
		Location faceLoc(Face::getTemplate(), Face::getBaseTag());
		i->fillInto(faceLoc);
		loc.add(faceLoc);
	}
}



std::string FaceList::getBaseTag()
{
	return "OBJECTS";
}



std::vector<Face> FaceList::getFaces()
{
	return faces;
}



void FaceList::addFace(Face face)
{
	faces.push_back(face);
}



void FaceList::sanitizeLocation(xmltio::Location & loc) const
{
	XPath xp(getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}


