/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

 /** ==========================================================================
 * 
 * @file		PersonDataList.cpp --
 * @brief		Source file for the list of person data.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#include "PersonDataList.h"

using namespace airobots;

PersonDataList::PersonDataList() {

}

PersonDataList::~PersonDataList() {

}

std::string 
PersonDataList::getBaseTag() {
	return "PERSONS";
}

std::string 
PersonDataList::getTemplate() {
	return "<" + getBaseTag() + ">" + "<GENERATOR/>"
		+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}

void 
PersonDataList::addPersonData(airobots::PersonData data) {
	personData.push_back(data);
}

std::vector<PersonData>
PersonDataList::getPersonData() {
	return personData;
}

int
PersonDataList::size() {
	return personData.size();
}

void
PersonDataList::sanitizeLocation(xmltio::Location &loc) const {

	xmltio::XPath xp(PersonData::getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}

}

void 
PersonDataList::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	int nrPersons = personData.size();
	for (int i = 0; i < nrPersons; i++) {
		xmltio::Location personLoc(airobots::PersonData::getTemplate(), airobots::PersonData::getBaseTag());
		personLoc = personData[i];
		loc.add(personLoc);		
	}

}

void 
PersonDataList::fromLocation(const xmltio::Location &loc, PersonDataList &list) {	

	Type::fromLocation(loc, list);

	for (xmltio::XPathIterator i = xmltio::XPath(airobots::PersonData::getBaseTag()).evaluate(loc); i; ++i) {
		xmltio::Location personLoc(*i);
		PersonData data = xmltio::extract<airobots::PersonData>(personLoc);
		list.addPersonData(data);
	}

}
