/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef ROBOTSTATE_H_
#define ROBOTSTATE_H_

#include "../../Type.h"

namespace airobots {

class RobotState: public Type {
public:

	enum RobotStates {
        /**
         * The robot is driving to a position
         */
        DRIVE_TO,
        /**
         * The robot shall speak to an interaction partner
         */
        TALK_TO,
        /**
         * The robot shall return to the HOME position, normally (0,0)
         */
        GO_HOME,
        /**
         * The robot is waiting for command somewhere
         */
        WAITING
	};

	RobotState();
	virtual ~RobotState();

	static std::string getBaseTag();
	static std::string getTemplate();

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, RobotState & rs);
	void sanitizeLocation(xmltio::Location &loc) const;

	virtual bool operator==(const RobotState &other) const;

	RobotStates getPreviousState() const;
	RobotStates getCurrentState() const;
	std::string getTargetName() const;

	void setPreviousState(RobotStates state);
	void setCurrentState(RobotStates state);
	void setTargetName(std::string target);

private:

	static const std::string CURRENT_STATE_ELEMENT_NAME;
	static const std::string PREVIOUS_STATE_ELEMENT_NAME;
	static const std::string STATE_TARGET_ELEMENT_NAME;

	RobotStates previousState;
	RobotStates currentState;
	std::string targetName;
};

}

#endif /* ROBOTSTATE_H_ */
