/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef REGIONDATA_H_
#define REGIONDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "../../Type.h"
#include "../../units/UnitConverter.h"
#include "../geometry/PrecisePolygon.h"

namespace airobots {

/**
 * Results of the object recognition.
 *
 * @author lziegler
 */
class RegionData: public Type {

public:

	enum Scope {
		LOCAL, GLOBAL
	};
	Scope scope;

	//std::vector<std::pair<double, double> > polygon;
	PrecisePolygon polygon;
	std::vector<std::pair<double, double> > scalePolygon(std::vector<std::pair<
			double, double> > poly, double factor) const;

public:

	static std::string getBaseTag();
	static std::string getTemplate();

	std::vector<std::pair<double, double> > getPolygon(LengthUnit unit) const;
	void setPolygon(const std::vector<std::pair<double, double> > &polygon,
			LengthUnit unit);

	PrecisePolygon getPolygon() const;
	void setPolygon(PrecisePolygon poly);

	void setScope(const Scope &scope);
	Scope getScope() const;

	virtual bool operator==(const RegionData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, RegionData &type);

	RegionData();
	RegionData(std::string generator);
};

}

#endif /* REGIONDATA_H_ */
