/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */
/*
 * PointCloudGrasping.h
 *
 *  Created on: Jun 14, 2011
 *      Author: plueckin
 */

#ifndef POINTCLOUDGRASPING_H_
#define POINTCLOUDGRASPING_H_

#include "../../Type.h"
#include "../geometry/Point3D.h"
#include "PointCloud.h"

namespace airobots {

class PointCloudGrasping: public Type {
public:
	PointCloudGrasping();
	PointCloudGrasping(std:: string generator);
	PointCloudGrasping(const PointCloudGrasping &pcg);
	PointCloudGrasping(PointCloud pc, Point3D gpnt, std::string lbl, double h,
			double w, double d, LengthUnit ilu);
	virtual ~PointCloudGrasping();

	static std::string getBaseTag();
	static std::string getTemplate();

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			PointCloudGrasping &type);
	void sanitizeLocation(xmltio::Location &loc) const;

	PointCloud getHull();
	Point3D getGraspPnt();
	std::string getLabel();
	double getHeight(LengthUnit lU);
	double getWidth(LengthUnit lU);
	double getDepth(LengthUnit lU);

	void setHull(PointCloud pc);
	void setGraspPnt(Point3D gpnt);
	void setLabel(std::string lbl);
	void setHeight(double h,LengthUnit lU);
	void setWidth(double w,LengthUnit lU);
	void setDepth(double d,LengthUnit lU);

	bool operator==(const PointCloudGrasping &other) const;

private:

	PointCloud convex_hull;
	Point3D grasppnt;
	std::string label;

	double height;
	double width;
	double depth;

	LengthUnit iLU;

};

}

#endif /* POINTCLOUDGRASPING_H_ */
