/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#ifndef SRCALIBRATEDDATA_H_
#define SRCALIBRATEDDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "../../Type.h"
#include "../geometry/Point3D.h"

namespace airobots {

/**
 * SwissRanger calibrated 3d data.
 * The points are positioned by rows. There is one Point3D object for each
 * pixel of the SwissRanger camera. Access to a specific row and col schould be
 * done like this:
 *
 * index = row * width + col
 *
 * @author lziegler
 */
class SRCalibratedData: public std::vector<Point3D>, public Type {

public:

	enum Scope {
		GLOBAL, LOCAL
	};

	static std::string getBaseTag();
	static std::string getTemplate();

	SRCalibratedData();
	SRCalibratedData(std::string generator);
	SRCalibratedData(std::string generator, SRCalibratedData::Scope scope);

	void setScope(SRCalibratedData::Scope scope);
	SRCalibratedData::Scope getScope() const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			SRCalibratedData &type);

private:
	SRCalibratedData::Scope scope;

};

}

#endif /* SRCALIBRATEDDATA_H_ */
