package de.unibi.airobots.btl.data.asu;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a node in a semantical word net. It combines a word with a frame
 * name (situated semantic unit, SSU).
 * 
 * @author malinke
 */
public class WordNode<T extends WordNode> {

	private final String word;

	private final String frameName;

	private final List<T> leaves;

	private String slotName;

	/**
	 * Creates a new node.
	 * 
	 * @param word
	 *            looked up in the lexicon
	 * @param frame
	 *            one meaning of the word
	 */
	public WordNode(final String word, final String frameName) {
		this.word = word;
		this.frameName = frameName;
		this.leaves = new ArrayList<T>();
	}

	/**
	 * Simply returns the word of this node.
	 * 
	 * @return looked up in the lexicon
	 */
	public final String getWord() {
		return word;
	}

	/**
	 * Adds a leave to the list of leaves to build a tree structure.
	 * 
	 * @param leave
	 *            to add
	 */
	public final void addLeave(T leave) {
		leaves.add(leave);
	}

	/**
	 * Provides the list of nodes connected to this.
	 * 
	 * @return direct connected nodes of the next level
	 */
	public final List<T> getLeaves() {
		return leaves;
	}

	/**
	 * Provides the name of the frame of this node.
	 * 
	 * @return name of the frame
	 */
	public final String getFrameName() {
		return frameName;
	}

	/**
	 * Provides the name of the slot, if set. If this node is the leave of
	 * another node, the slot is the required frame name of the slot of the
	 * parent node.
	 * 
	 * @return the slot name, if this node has a parent, null otherwise
	 */
	public final String getSlotName() {
		return slotName;
	}

	/**
	 * Sets the slot name. Has to be called, if this node is connected as leave.
	 * 
	 * @param name
	 *            The name of the slot, where this node fits into.
	 */
	public final void setSlotName(String name) {
		slotName = name;
	}

}
