package de.unibi.airobots.btl.data.taskpattern;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.data.common.ElementParser;
import de.unibi.airobots.btl.data.common.UnexpectedElementFormatException;

/**
 * Send by the dialog to the system. system should handle this.
 * @author arothert, vrichter, bcarlmey
 */
public class Say extends TaskPatternBase {

	static final String UTTERANCE_ELEM = "UTTERANCE";
	static final String LENGTH_ELEM = "length";


	private String utterance;
	private int length;
	
	public Say() {	
	}
	
	/**
	 * the base tag should be named after the class itself.
	 * @return
	 */
	public static String getBaseTag() {
		return "SAY";
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		Element utteranceElement = new Element(UTTERANCE_ELEM, utterance);
		Element statesElement = 
			TaskPatternStatusTagData.createTaskStatusElement(getOrigin(), getValue());
		Element lengthElement = new Element(LENGTH_ELEM, ("" + length));
		parent.appendChild(utteranceElement);
		parent.appendChild(statesElement);
		parent.appendChild(lengthElement);
	}
		
	public static void fromElement(Element element, Say type) {
		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals(UTTERANCE_ELEM)) {
					try {
						type.setUtterance(ElementParser.getTagValue(elem));
					} catch (UnexpectedElementFormatException e) {
						type.setUtterance("");
					}
				}
				if (elem.getLocalName().equals(STATUS_TAG)) {
					type.setValue(ElementParser.getAttributeValue(elem, VALUE_ATT));
					type.setOrigin(ElementParser.getAttributeValue(elem, ORIGIN_ATT));	
				}
				if (elem.getLocalName().equals(LENGTH_ELEM)) {
					try {
						type.setLength(ElementParser.getTagValue(elem));
					} catch (UnexpectedElementFormatException e) {
						type.setLength(0);
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("["+getBaseTag()+": ");
		strb.append("[UTTERANCE: " + utterance + "]");
		strb.append("[STATUS: ");
		strb.append("value: " + value + "; ");
		strb.append("origin: " + origin + "]");
		strb.append("[length: " + length + "]]");
		return strb.toString();
	}
	
	public String getUtterance() {
		return utterance;
	}

	public void setUtterance(String utterance) {
		this.utterance = utterance;
	}

	public int getLength() {
		return length;
	}

	public void setLength(int length) {
		this.length = length;
	}
	
	public void setLength(String length) {
		this.length = Integer.parseInt(length);
	}
}
