/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * AnnotationTest.cpp
 *
 *  Created on: April 18, 2012
 *      Author: lkettenb
 */

#include <boost/test/unit_test.hpp>
#include "btl/data/map/Annotation.h"

BOOST_AUTO_TEST_SUITE(AnnotationTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try {
        
        airobots::Annotation annotation;
        annotation.setLabel("kitchen");
        airobots::PrecisePolygon polygon;
        polygon.addPoint(0.0, 0.0, airobots::METER);
        polygon.addPoint(0.0, 2.0, airobots::METER);
        polygon.addPoint(2.0, 0.0, airobots::METER);
        polygon.addPoint(2.0, 2.0, airobots::METER);
        annotation.setPolygon(polygon);
        
        airobots::Viewpoint viewpoint1;
        viewpoint1.setLabel("Bottle0");
        viewpoint1.setCategory(airobots::Viewpoint::OBJECT);
        std::list<airobots::Viewpoint> viewpoints;
        viewpoints.push_back(viewpoint1);
        
        airobots::Viewpoint viewpoint2 = viewpoint1;
        viewpoint2.setLabel("Bottle1");
        viewpoints.push_back(viewpoint2);
        annotation.setViewpoints(viewpoints);
        
        xmltio::Location loc(airobots::Annotation::getTemplate(), airobots::Annotation::getBaseTag());
		loc = annotation;
        airobots::Annotation parsedAnnotation = xmltio::extract<airobots::Annotation>(loc);
        
        BOOST_REQUIRE_EQUAL(parsedAnnotation.getLabel() == "kitchen", true);
        BOOST_REQUIRE_EQUAL(parsedAnnotation.getViewpoints()->back().getLabel() == "Bottle1", true);
	} catch (std::exception e) {
		BOOST_FAIL(std::string("Caught exception: ") + e.what());
	}
}
BOOST_AUTO_TEST_SUITE_END()
