/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>
#include "boost/random.hpp"
#include "boost/generator_iterator.hpp"

#include <iostream>

#include "btl/data/navigation/ROSMoveBaseConfiguration.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ROSMoveBaseConfigurationTest)
        
BOOST_AUTO_TEST_CASE(testSelfCompatibility) {
    
    typedef boost::mt19937 RNGType;
    RNGType rng(time(0)); 
    boost::uniform_real<> one_to_fifty( 0.0, 50.0 );
    boost::variate_generator< RNGType, boost::uniform_real<> >
            rnd(rng, one_to_fifty);

    try {
        ROSMoveBaseConfiguration original = ROSMoveBaseConfiguration();
        original.setGenerator("ROSMaveBaseConfigTest");
        original.setTimestamp(Timestamp(123, 456));
        original.setBaseGlobalPlanner("navfn/NavfnROS");
	original.setBaseLocalPlanner("base_local_planner/TrajectoryPlannerROS");
        original.setClearingRotationAllowed(rnd() > rnd());
        original.setConservativeResetDist(rnd());
        original.setControllerFrequency(rnd());
        original.setControllerPatience(rnd());
        original.setOscillationDistance(rnd());
        original.setOscillationTimeout(rnd());
        original.setPlannerFrequency(rnd());   
        original.setPlannerPatience(rnd());
        original.setRecoveryBehaviorEnabled(rnd() > rnd());                
        original.setRestoreDefaults(rnd() > rnd());
        original.setShutdownCostmaps(rnd() > rnd());    
        
        Location originalLoc(
            ROSMoveBaseConfiguration::getTemplate(), 
            ROSMoveBaseConfiguration::getBaseTag());

        originalLoc = original;

        ROSMoveBaseConfiguration parsed = extract<ROSMoveBaseConfiguration>(originalLoc);
        
        BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
        BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
        BOOST_REQUIRE_EQUAL(parsed.getBaseGlobalPlanner(), original.getBaseGlobalPlanner());
        BOOST_REQUIRE_EQUAL(parsed.getBaseLocalPlanner(), original.getBaseLocalPlanner());        
        BOOST_REQUIRE_EQUAL(parsed.isClearingRotationAllowed(), original.isClearingRotationAllowed());        
        BOOST_REQUIRE_CLOSE(parsed.getConservativeResetDist(), original.getConservativeResetDist(), TOLERANCE);
        BOOST_REQUIRE_CLOSE(parsed.getControllerFrequency(), original.getControllerFrequency(), TOLERANCE);
        BOOST_REQUIRE_CLOSE(parsed.getControllerPatience(), original.getControllerPatience(), TOLERANCE);
        BOOST_REQUIRE_CLOSE(parsed.getOscillationDistance(), original.getOscillationDistance(), TOLERANCE);
        BOOST_REQUIRE_CLOSE(parsed.getOscillationTimeout(), original.getOscillationTimeout(), TOLERANCE);                  
        BOOST_REQUIRE_CLOSE(parsed.getPlannerFrequency(), original.getPlannerFrequency(), TOLERANCE);
        BOOST_REQUIRE_CLOSE(parsed.getPlannerPatience(), original.getPlannerPatience(), TOLERANCE);
        BOOST_REQUIRE_EQUAL(parsed.isRestoreDefaults(), original.isRestoreDefaults());
        BOOST_REQUIRE_EQUAL(parsed.isRecoveryBehaviorEnabled(), original.isRecoveryBehaviorEnabled());
        BOOST_REQUIRE_EQUAL(parsed.isShutdownCostmaps(), original.isShutdownCostmaps());
    } catch (std::exception e) {
            BOOST_FAIL(string("Caught exception: ") + e.what());
    }
}

BOOST_AUTO_TEST_SUITE_END()
