/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/data/object/Door.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

BOOST_AUTO_TEST_SUITE(DoorTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		Door original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setDoorLeft(Point2D(1.0, 2.0, METER));
		original.setDoorRight(Point2D(3.0, 4.0, METER));
		original.setHandleLeft(Point3D(5.0, 6.0, 7.0, METER));
		original.setHandleRight(Point3D(8.0, 9.0, 0.0, METER));
		original.setCoordinateSystem(Door::RobotCS);
		original.setConfidence(0.123);
		Location originalLoc(Door::getTemplate(), Door::getBaseTag());
		originalLoc = original;

		Door parsed = extract<Door>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());

		BOOST_REQUIRE_EQUAL(parsed, original);
		parsed.setGenerator("trash");
		BOOST_REQUIRE(!(parsed == original));
		parsed.setGenerator(original.getGenerator());
		parsed.setDoorRight(Point2D(0.0, 0.0, METER));
		BOOST_REQUIRE(!(parsed == original));

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Point2D p0(1.0, 2.0, METER);
	Point2D p1(3.0, 4.0, METER);
	Point3D p2(5.0, 6.0, 7.0, METER);
	Point3D p3(8.0, 9.0, 0.0, METER);

	p0.setTimestamp(Timestamp(123, 456));
	p1.setTimestamp(Timestamp(123, 456));
	p2.setTimestamp(Timestamp(123, 456));
	p3.setTimestamp(Timestamp(123, 456));

	Door original;
	original.setGenerator("test");
	original.setTimestamp(Timestamp(123, 456));
	original.setDoorLeft(p0);
	original.setDoorRight(p1);
	original.setHandleLeft(p2);
	original.setHandleRight(p3);
	original.setCoordinateSystem(Door::RobotCS);
	original.setConfidence(0.123);

	Location fileContents(readFile("Door.xml"), Door::getBaseTag());
	Door parsed = extract<Door>(fileContents);

	string name = Door::getPerceptName();

	LengthUnit u = METER;

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_CLOSE(original.getDoorLeft().getX(u),
			original.getDoorLeft().getX(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getDoorLeft().getY(u),
			original.getDoorLeft().getY(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getDoorRight().getX(u),
			original.getDoorRight().getX(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getDoorRight().getY(u),
			original.getDoorRight().getY(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getHandleLeft().getX(u),
			original.getHandleLeft().getX(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getHandleLeft().getY(u),
			original.getHandleLeft().getY(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getHandleRight().getX(u),
			original.getHandleRight().getX(u), 0.0001);
	BOOST_REQUIRE_CLOSE(original.getHandleRight().getY(u),
			original.getHandleRight().getY(u), 0.0001);
	BOOST_REQUIRE(
			original.getCoordinateSystem() == parsed.getCoordinateSystem());
	BOOST_REQUIRE( original.getConfidence() == parsed.getConfidence());

}
BOOST_AUTO_TEST_SUITE_END()
