/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

/*
 * PointCloudGraspingTest.cpp
 *
 *  Created on: Jun 16, 2011
 *      Author: plueckin
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>
#include <math.h>

#include "btl/data/vision3d/PointCloudGrasping.h"
#include "btl/units/LengthUnit.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PointCloudGraspingTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {
		PointCloud original_pc;
		PointCloudGrasping original_pcg;

		Point3D p0(4.2, 1.4, 2.4, MILLIMETER);
		Point3D p1(2.7, 4.3, 7.8, MILLIMETER);
		Point3D p2(6.4, 3.9, 5.0, MILLIMETER);

		original_pc.addPoint(p0);
		original_pc.addPoint(p1);
		original_pc.addPoint(p2);

		Point3D ori(0, -1, 0, MILLIMETER);
		original_pc.setSensorOrigin(ori);

		Rotation3D rot(1, 2, 3, 4, RADIAN);
		original_pc.setRotation(rot);

		original_pcg.setHeight(2.5,MILLIMETER);
		original_pcg.setWidth(1.3,MILLIMETER);
		original_pcg.setDepth(4.1,MILLIMETER);

		Point3D g0(4.4, 1.9, 4.31, MILLIMETER);

		original_pcg.setLabel("pringles");
		original_pcg.setGraspPnt(g0);
		original_pcg.setHull(original_pc);

		Location originalLoc(PointCloudGrasping::getTemplate(), PointCloudGrasping::getBaseTag());

		originalLoc = original_pcg;

		PointCloudGrasping parsed = extract<PointCloudGrasping>(originalLoc);

		BOOST_REQUIRE_EQUAL(original_pcg.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original_pcg.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_EQUAL(original_pcg.getLabel(), parsed.getLabel());
		BOOST_REQUIRE_CLOSE(original_pcg.getGraspPnt().getX(), parsed.getGraspPnt().getX(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original_pcg.getGraspPnt().getY(), parsed.getGraspPnt().getY(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original_pcg.getGraspPnt().getZ(), parsed.getGraspPnt().getZ(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original_pcg.getHull().getPoints()[0].getX(), parsed.getHull().getPoints()[0].getX(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original_pcg.getHull().getPoints()[1].getY(), parsed.getHull().getPoints()[1].getY(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original_pcg.getHull().getPoints()[2].getZ(), parsed.getHull().getPoints()[2].getZ(), TOLERANCE);

		BOOST_REQUIRE(original_pcg == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("PointCloudGrasping.xml"), PointCloudGrasping::getBaseTag());
	PointCloudGrasping parsed = extract<PointCloudGrasping>(fileContents);

	BOOST_REQUIRE_EQUAL("PointCloudGraspingTester", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(666, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL("pringles", parsed.getLabel());
	BOOST_REQUIRE_CLOSE(3.1, parsed.getHeight(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.2, parsed.getWidth(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.3, parsed.getDepth(MILLIMETER), TOLERANCE);

	BOOST_REQUIRE_CLOSE(12.2, parsed.getGraspPnt().getX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(43.222, parsed.getGraspPnt().getY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(0.456, parsed.getGraspPnt().getZ(MILLIMETER), TOLERANCE);
}

BOOST_AUTO_TEST_SUITE_END()
