package de.unibi.airobots.btl;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;

import java.io.IOException;
import java.lang.reflect.Method;

import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.junit.BeforeClass;
import org.junit.Test;

import de.unibi.airobots.btl.List;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.person.Face;

/**
 * Test cases for {@link Type}.
 */
public class ListTest {

	private Logger logger = Logger.getLogger(getClass());
	
	@BeforeClass
	public static void setUp() {
		BasicConfigurator.configure();
	}

	@Test
	public void selfCompatibility() throws Exception {

		final long timestamp = System.currentTimeMillis();

		List<Face> original = new List<Face>(Face.class);
		original.setGenerator("test");
		original.setTimestamp(timestamp);

		Face face1 = new Face();
		face1.setGenerator("facesensor1");
		face1.setTimestamp(timestamp);
		face1.setClassId(1);
		original.add(face1);

		Face face2 = new Face();
		face2.setGenerator("facesensor2");
		face2.setTimestamp(timestamp);
		face2.setClassId(2);
		original.add(face2);

		// Document doc = new Document(new Element(List.getBaseTag()));
		Document doc = new Document(new Element(List.getBaseTag(Face.class)));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		List<Face> parsed = new List<Face>(Face.class);
		List.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.size(), parsed.size());
		assertEquals(original.get(0).getClassId(), parsed.get(0).getClassId());

	}

	@Test
	public void testDynamicInstantiation() throws Exception {

		final long timestamp = System.currentTimeMillis();

		List<Face> original = new List<Face>(Face.class);
		original.setGenerator("test");
		original.setTimestamp(timestamp);

		Face face1 = new Face();
		face1.setGenerator("facesensor1");
		face1.setTimestamp(timestamp);
		face1.setClassId(1);
		original.add(face1);

		Face face2 = new Face();
		face2.setGenerator("facesensor2");
		face2.setTimestamp(timestamp);
		face2.setClassId(2);
		original.add(face2);

		// Document doc = new Document(new Element(List.getBaseTag()));
		Document doc = new Document(new Element(List.getBaseTag(Face.class)));
		original.fillInto(doc.getRootElement());

		Class<List<Face>> typeClass = (Class<List<Face>>) original.getClass();

		Method fromDocumentMethod = typeClass.getMethod("fromDocument",
				new Class[] { Document.class, Class.class });

		Object rawResult = fromDocumentMethod.invoke(null, doc, typeClass);

		if (rawResult == null) {
			fail("fromDocument method has wrong signature "
					+ "or simply returned null.");
		}

		List<Face> parsed = (List<Face>) rawResult;

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.size(), parsed.size());
		assertEquals(original.get(0).getClassId(), parsed.get(0).getClassId());
	}
}
