package de.unibi.airobots.btl.data.command;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.data.speechrec.GrammarNonTerminal;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.ParsingException;
import org.apache.log4j.Logger;
import static org.junit.Assert.assertEquals;
import org.junit.Test;

/**
 *
 * @author ikillman
 */
public class CommandDataTest {
    
    private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws ParsingException {

            CommandData command = new CommandData();
            final long timestamp = System.currentTimeMillis();
            command.setTimestamp(timestamp);
            command.setAction("bring");
            command.setLocation("appliance");
            command.setObject("drink");
            GrammarNonTerminal tree = new GrammarNonTerminal();
            command.setTree(tree);

            Document document = new Document(new Element(CommandData.getBaseTag()));
            command.fillInto(document.getRootElement());

            logger.info(document.toXML());

            CommandData parsed = new CommandData();
            CommandData.fromElement(document.getRootElement(), parsed);


            assertEquals(command.getAction(), parsed.getAction());
            assertEquals(command.getLocation(), parsed.getLocation());
            assertEquals(command.getObject(), parsed.getObject());
            assertEquals(command.getTree(), parsed.getTree());
		
	}

	@Test
	public void fileCompatibility() throws Exception {
		
            CommandData command = new CommandData();
            final long timestamp = System.currentTimeMillis();
            command.setTimestamp(timestamp);
            command.setAction("bring");
            command.setLocation("appliance");
            command.setObject("drink");
            GrammarNonTerminal tree = new GrammarNonTerminal();
            command.setTree(tree);

            Builder parser = new Builder();
            Document doc = parser.build(TestUtils.makeTestFileName("CommandData.xml"));

            CommandData parsed = new CommandData();
            CommandData.fromElement(doc.getRootElement(), parsed);

            assertEquals(doc.getRootElement().getLocalName(), CommandData.getBaseTag());
            assertEquals("test", parsed.getGenerator());

            assertEquals(command.getAction(), parsed.getAction());
            assertEquals(command.getLocation(), parsed.getLocation());
            assertEquals(command.getObject(), parsed.getObject());
            assertEquals(command.getTree(), parsed.getTree());
            

	}
    
}
