package de.unibi.airobots.btl.data.grasp;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Node;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.grasp.KatanaGripperData;

/**
 * Test cases for {@link Type}.
 * 
 * @author ttoenige
 */
public class KatanaGripperDataTest {
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		KatanaGripperData original = new KatanaGripperData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setForceRightInsideNear(1);
		original.setForceRightInsideFar(2);
		original.setInfraredRightOutside(3);
		original.setInfraredRightFront(4);
		original.setInfraredRightInsideNear(5);
		original.setInfraredRightInsideFar(6);
		original.setForceLeftInsideNear(7);
		original.setForceLeftInsideFar(8);
		original.setInfraredLeftOutside(9);
		original.setInfraredLeftFront(10);
		original.setInfraredLeftInsideNear(11);
		original.setInfraredLeftInsideFar(12);
		original.setInfraredMiddle(13);

		Element test = new Element(KatanaGripperData.getBaseTag());
		Document doc = new Document(test);
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		KatanaGripperData parsed = new KatanaGripperData();
		KatanaGripperData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getForceRightInsideNear(), parsed
				.getForceRightInsideNear(), 0.00001);
		assertEquals(original.getForceRightInsideFar(), parsed
				.getForceRightInsideFar(), 0.00001);
		assertEquals(original.getInfraredRightOutside(), parsed
				.getInfraredRightOutside(), 0.00001);
		assertEquals(original.getInfraredRightFront(), parsed
				.getInfraredRightFront(), 0.00001);
		assertEquals(original.getInfraredRightInsideNear(), parsed
				.getInfraredRightInsideNear(), 0.00001);
		assertEquals(original.getInfraredRightInsideFar(), parsed
				.getInfraredRightInsideFar(), 0.00001);
		assertEquals(original.getForceLeftInsideNear(), parsed
				.getForceLeftInsideNear(), 0.00001);
		assertEquals(original.getForceLeftInsideFar(), parsed
				.getForceLeftInsideFar(), 0.00001);
		assertEquals(original.getInfraredLeftOutside(), parsed
				.getInfraredLeftOutside(), 0.00001);
		assertEquals(original.getInfraredLeftFront(), parsed
				.getInfraredLeftFront(), 0.00001);
		assertEquals(original.getInfraredLeftInsideNear(), parsed
				.getInfraredLeftInsideNear(), 0.00001);
		assertEquals(original.getInfraredLeftInsideFar(), parsed
				.getInfraredLeftInsideFar(), 0.00001);
		assertEquals(original.getInfraredMiddle(), parsed.getInfraredMiddle(),
				0.00001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("KatanaGripperData.xml"));

		KatanaGripperData parsed = new KatanaGripperData();
		KatanaGripperData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), KatanaGripperData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(1, parsed.getForceRightInsideNear(), 0.00001);
		assertEquals(2, parsed.getForceRightInsideFar(), 0.00001);
		assertEquals(3, parsed.getInfraredRightOutside(), 0.00001);
		assertEquals(4, parsed.getInfraredRightFront(), 0.00001);
		assertEquals(5, parsed.getInfraredRightInsideNear(), 0.00001);
		assertEquals(6, parsed.getInfraredRightInsideFar(), 0.00001);
		assertEquals(7, parsed.getForceLeftInsideNear(), 0.00001);
		assertEquals(8, parsed.getForceLeftInsideFar(), 0.00001);
		assertEquals(9, parsed.getInfraredLeftOutside(), 0.00001);
		assertEquals(10, parsed.getInfraredLeftFront(), 0.00001);
		assertEquals(11, parsed.getInfraredLeftInsideNear(), 0.00001);
		assertEquals(12, parsed.getInfraredLeftInsideFar(), 0.00001);
		assertEquals(13, parsed.getInfraredMiddle(), 0.00001);

	}

}
