package de.unibi.airobots.btl.data.grasp;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.grasp.KatanaPoseData;
import de.unibi.airobots.btl.units.*;

/**
 * Test cases for {@link Type}.
 * 
 * @author ttoenige
 */
public class KatanaPoseDataTest {
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		KatanaPoseData original = new KatanaPoseData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setX(111,LengthUnit.MILLIMETER);
		original.setY(222,LengthUnit.MILLIMETER);
		original.setZ(333,LengthUnit.MILLIMETER);
		original.setPhi(1.11,AngleUnit.RADIAN);
		original.setPsi(0.55,AngleUnit.RADIAN);
		original.setTheta(0.77,AngleUnit.RADIAN);

		Document doc = new Document(new Element(KatanaPoseData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		KatanaPoseData parsed = new KatanaPoseData();
		KatanaPoseData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getX(LengthUnit.MILLIMETER), parsed.getX(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(original.getY(LengthUnit.MILLIMETER), parsed.getY(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(original.getZ(LengthUnit.MILLIMETER), parsed.getZ(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(original.getPhi(AngleUnit.RADIAN), parsed.getPhi(AngleUnit.RADIAN), 0.00001);
		assertEquals(original.getPsi(AngleUnit.RADIAN), parsed.getPsi(AngleUnit.RADIAN), 0.00001);
		assertEquals(original.getTheta(AngleUnit.RADIAN), parsed.getTheta(AngleUnit.RADIAN), 0.00001);

	}
	
	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("KatanaPoseData.xml"));

		KatanaPoseData parsed = new KatanaPoseData();
		KatanaPoseData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), KatanaPoseData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(232, parsed.getX(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(343, parsed.getY(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(454, parsed.getZ(LengthUnit.MILLIMETER), 0.00001);
		assertEquals(1.23, parsed.getPsi(AngleUnit.RADIAN), 0.00001);
		assertEquals(1.34, parsed.getPhi(AngleUnit.RADIAN), 0.00001);
		assertEquals(0.23, parsed.getTheta(AngleUnit.RADIAN), 0.00001);
		

	}


}
