package de.unibi.airobots.btl.data.navigation;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.MicroTimestamp;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.navigation.OdometryData;
import de.unibi.airobots.btl.units.*;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class OdometryDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		OdometryData original = new OdometryData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setX(-12.34,LengthUnit.MILLIMETER);
		original.setY(34123.45,LengthUnit.MILLIMETER);
		original.setYaw(1.234,AngleUnit.RADIAN);

		Document doc = new Document(new Element(OdometryData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		OdometryData parsed = new OdometryData();
		OdometryData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getX(LengthUnit.MILLIMETER), parsed.getX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getY(LengthUnit.MILLIMETER), parsed.getY(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getYaw(AngleUnit.RADIAN), parsed.getYaw(AngleUnit.RADIAN), 0.0001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("RobotPosition.xml"));

		OdometryData parsed = new OdometryData();
		OdometryData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), OdometryData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(234.5, parsed.getX(LengthUnit.METER), 0.00001);
		assertEquals(-6345.42, parsed.getY(LengthUnit.METER), 0.00001);
		assertEquals(0.34, parsed.getYaw(AngleUnit.RADIAN), 0.00001);
		assertEquals(new MicroTimestamp(123213412, 335435), parsed
				.getReadTime());

	}

}
