package de.unibi.airobots.btl.data.speechrec;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.assertEquals;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.speechrec.GPSRAction.GPSRParameter;

/**
 * Test cases for {@link Type}.
 * 
 * @author hterhors
 */
public class GPSRActionTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		GPSRAction original = new GPSRAction();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setName("putObject");
		GPSRParameter param1 = original.new GPSRParameter("cup", "object");
		GPSRParameter param2 = original.new GPSRParameter("table", "kitchen");

		original.addParam(param1);
		original.addParam(param2);

		Document doc = new Document(new Element(GPSRAction.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		GPSRAction parsed = new GPSRAction();
		GPSRAction.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getName(), parsed.getName());
		assertEquals(original.getParams().get(0).getName(), parsed.getParams()
				.get(0).getName());
		assertEquals(original.getParams().get(1).getName(), parsed.getParams()
				.get(1).getName());
		assertEquals(original.getParams().get(0).getType(), parsed.getParams()
				.get(0).getType());
		assertEquals(original.getParams().get(1).getType(), parsed.getParams()
				.get(1).getType());
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("GPSRActionTest.xml"));

		GPSRAction parsed = new GPSRAction();
		GPSRAction.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(),
				GPSRAction.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals("putObject", parsed.getName());
		assertEquals("cup", parsed.getParams().get(0).getName());
		assertEquals("table", parsed.getParams().get(1).getName());
		assertEquals("object", parsed.getParams().get(0).getType());
		assertEquals("location", parsed.getParams().get(1).getType());
	}

}
