package de.unibi.airobots.btl.data.vision2d;

/*
 * #LICENSE%
 * BielefeldTypeLibrary
 * ---
 * Copyright (C) 2009 - 2013 Frederic Siepmann
 * ---
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 * 
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 * 
 * Contributors: Florian Lier, Frederic Siepmann, Leon Ziegler, Matthias 
 * Schoepfer, Adriana-Victoria Dreyer, Agnes Swadzba, Andreas Kipp, Birte 
 * Carlmeyer, Christian Ascheberg, Daniel Nacke, Dennis Wigand, Günes Minareci, 
 * Hendrik ter Horst, Ingo Killmann, Jan Pöppel, Lukas Kettenbach, Michael 
 * Zeunert, Patrick Renner, Philipp Dresselhaus, Sebastian Meyer zu Borgsen, 
 * Soufian Jebbara, Tobias Röhlig, Torben Toeniges, Viktor Losing, Viktor Richter
 * %LICENSE#
 */

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.data.vision2d.RegionData;
import de.unibi.airobots.btl.data.vision2d.RegionData.Scope;
import de.unibi.airobots.btl.units.LengthUnit;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class RegionDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		RegionData original = new RegionData();
		original.setScope(Scope.LOCAL);

		Document doc = new Document(new Element(RegionData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		RegionData parsed = new RegionData();
		RegionData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getScope(), parsed.getScope());
		assertEquals(original.getPolygon(), parsed.getPolygon());

		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("RegionData.xml"));

		RegionData parsed = new RegionData();
		RegionData.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(), RegionData
				.getBaseTag());

		assertEquals(new Timestamp(253543797l, 253543712l), parsed
				.getTimestamp());
		assertEquals(Scope.LOCAL, parsed.getScope());
		//assertEquals(poly, parsed.getPolygon());
	}
}
